import os
from typing import Any
from typing import Dict
from typing import List
from typing import Mapping
from typing import Optional

from mypy_extensions import TypedDict
from service_configuration_lib import read_service_configuration


class BaseSecretProvider:
    def __init__(
        self,
        soa_dir: Optional[str],
        service_name: Optional[str],
        cluster_names: List[str],
        **kwargs: Any,
    ) -> None:
        self.soa_dir = soa_dir
        self.cluster_names = cluster_names
        self.service_name = service_name
        if service_name:
            self.secret_dir = os.path.join(self.soa_dir, self.service_name, "secrets")
            service_config = read_service_configuration(self.service_name, self.soa_dir)
            self.encryption_key = service_config.get("encryption_key", "paasta")

    def decrypt_environment(
        self, environment: Dict[str, str], **kwargs: Any
    ) -> Dict[str, str]:
        raise NotImplementedError

    def write_secret(
        self,
        action: str,
        secret_name: str,
        plaintext: bytes,
        cross_environment_motivation: Optional[str] = None,
    ) -> None:
        raise NotImplementedError

    def decrypt_secret(self, secret_name: str) -> str:
        raise NotImplementedError

    def decrypt_secret_raw(self, secret_name: str) -> bytes:
        raise NotImplementedError

    def get_secret_signature_from_data(self, data: Mapping[str, Any]) -> Optional[str]:
        raise NotImplementedError

    def get_data_from_vault_path(self, path: str) -> Optional[Dict[str, str]]:
        raise NotImplementedError


class CryptoKey(TypedDict):
    key_name: str
    key_version: str
    key: str


class SecretProvider(BaseSecretProvider):
    def get_key_versions(self, key_name: str) -> List[CryptoKey]:
        """
        Dummy attribute to satisfy `mypy` because the class is imported dynamically via __import__
        """
        raise NotImplementedError



#!/usr/bin/env python
# Copyright 2015-2016 Yelp Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import argparse
import logging
import sys
from socket import getfqdn

from paasta_tools import mesos_maintenance

log = logging.getLogger(__name__)


def parse_args():
    """Parses the command line arguments passed to this script"""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-d",
        "--duration",
        type=mesos_maintenance.parse_timedelta,
        default="1h",
        help="Duration of the maintenance window. Any pytimeparse unit is supported.",
    )
    parser.add_argument(
        "-s",
        "--start",
        type=mesos_maintenance.parse_datetime,
        default=str(mesos_maintenance.now()),
        help="Time to start the maintenance window. Defaults to now.",
    )
    parser.add_argument(
        "action",
        choices=[
            "cluster_status",
            "down",
            "drain",
            "is_host_down",
            "is_host_drained",
            "is_host_draining",
            "is_hosts_past_maintenance_end",
            "is_hosts_past_maintenance_start",
            "is_safe_to_kill",
            "schedule",
            "status",
            "undrain",
            "up",
        ],
        help="Action to perform on the specified hosts",
    )
    parser.add_argument(
        "hostname",
        nargs="*",
        default=[getfqdn()],
        help="Hostname(s) of machine(s) to start draining. "
        "You can specify <hostname>|<ip> to avoid querying DNS to determine the corresponding IP.",
    )
    parser.add_argument(
        "-v",
        "--verbose",
        action="count",
        dest="verbose",
        default=0,
        help="Print out more output.",
    )
    return parser.parse_args()


def is_safe_to_kill(hostname):
    """Checks if a host has drained or reached its maintenance window
    :param hostname: hostname to check
    :returns: True or False
    """
    return mesos_maintenance.is_host_drained(
        hostname
    ) or mesos_maintenance.is_host_past_maintenance_start(hostname)


def paasta_maintenance():
    """Manipulate the maintenance state of a PaaSTA host.
    :returns: None
    """
    args = parse_args()

    if args.verbose >= 2:
        logging.basicConfig(level=logging.DEBUG)
    elif args.verbose == 1:
        logging.basicConfig(level=logging.INFO)
    else:
        logging.basicConfig(level=logging.WARNING)

    action = args.action
    hostnames = args.hostname

    if action != "status" and not hostnames:
        print("You must specify one or more hostnames")
        return

    start = args.start
    duration = args.duration

    ret = "Done"
    if action == "drain":
        mesos_maintenance.drain(hostnames, start, duration)
    elif action == "undrain":
        mesos_maintenance.undrain(hostnames)
    elif action == "down":
        mesos_maintenance.down(hostnames)
    elif action == "up":
        mesos_maintenance.up(hostnames)
    elif action == "status":
        ret = mesos_maintenance.friendly_status()
    elif action == "cluster_status":
        ret = mesos_maintenance.status()
    elif action == "schedule":
        ret = mesos_maintenance.schedule()
    elif action == "is_safe_to_kill":
        ret = is_safe_to_kill(hostnames[0])
    elif action == "is_host_drained":
        ret = mesos_maintenance.is_host_drained(hostnames[0])
    elif action == "is_host_down":
        ret = mesos_maintenance.is_host_down(hostnames[0])
    elif action == "is_host_draining":
        ret = mesos_maintenance.is_host_draining(hostnames[0])
    elif action == "is_host_past_maintenance_start":
        ret = mesos_maintenance.is_host_past_maintenance_start(hostnames[0])
    elif action == "is_host_past_maintenance_end":
        ret = mesos_maintenance.is_host_past_maintenance_end(hostnames[0])
    else:
        raise NotImplementedError("Action: '%s' is not implemented." % action)
    print(ret)
    return ret


if __name__ == "__main__":
    if paasta_maintenance():
        sys.exit(0)
    sys.exit(1)
